import os
import sqlalchemy as sa

from datetime import datetime
from sqlalchemy.orm import declarative_base
from tic.shared.apc.enums.apc_category import ApcCategory

Base = declarative_base(metadata=sa.MetaData(schema=os.getenv("MFG_DB_SCHEMA", "dbo")))


class UpdateHashes(Base):
    __tablename__ = "update_hashes"
    id = sa.Column(sa.BigInteger, primary_key=True)
    sha256 = sa.Column(sa.LargeBinary, nullable=False, unique=True)
    update_hash = sa.Column(sa.Integer, nullable=False)


class FeedLog(Base):
    __tablename__ = "feed_log"
    id = sa.Column(sa.BigInteger, primary_key=True)
    mfg_last_processed_id = sa.Column(sa.BigInteger, nullable=False)
    bit9_last_processed_id = sa.Column(sa.BigInteger, nullable=False)
    last_processed_date_modified = sa.Column(
        sa.DateTime, nullable=False, default=datetime.utcnow()
    )


class HashInsertsAndUpdate(Base):
    __tablename__ = "hash_inserts_and_update"
    id = sa.Column(sa.BigInteger, primary_key=True)
    sha256 = sa.Column(sa.LargeBinary, nullable=False)
    date_modified = sa.Column(sa.DateTime, nullable=False, default=datetime.utcnow())


class HashIndex(Base):
    __tablename__ = "hash_index"
    id = sa.Column(sa.BigInteger, primary_key=True)
    md5 = sa.Column(sa.LargeBinary)
    md5_id = sa.Column(sa.SmallInteger, nullable=False, default=0)
    sha1 = sa.Column(sa.LargeBinary)
    sha1_id = sa.Column(sa.SmallInteger, nullable=False, default=0)
    sha256 = sa.Column(sa.LargeBinary, nullable=False)
    date_created = sa.Column(sa.DateTime, nullable=False, default=datetime.utcnow())
    mfg_id = sa.Column(sa.BigInteger)


class Mfg(Base):
    __tablename__ = "mfg"
    id = sa.Column(sa.BigInteger, primary_key=True)
    malicious = sa.Column(sa.Numeric, nullable=False, default=0)
    platform_num_scanner_votes = sa.Column(sa.SmallInteger, nullable=False, default=0)
    platform_classification_detail = sa.Column(
        sa.SmallInteger, nullable=False, default=0
    )
    malware_family_num_scanner_votes = sa.Column(
        sa.SmallInteger, nullable=False, default=0
    )
    malware_family_classification_detail = sa.Column(
        sa.SmallInteger, nullable=False, default=0
    )
    malware_group_num_scanner_votes = sa.Column(
        sa.SmallInteger, nullable=False, default=0
    )
    malware_group_classification_detail = sa.Column(
        sa.SmallInteger, nullable=False, default=0
    )
    date_modified = sa.Column(sa.DateTime, nullable=False, default=datetime.utcnow())
    latest_scan_date = sa.Column(sa.DateTime, nullable=False, default=datetime.utcnow())
    times_updated = sa.Column(sa.Integer, nullable=False, default=0)
    platform_id = sa.Column(sa.Integer)
    malware_family_id = sa.Column(sa.Integer)
    malware_group_id = sa.Column(sa.Integer)
    additional_info_id = sa.Column(sa.BigInteger)
    mfg_version_id = sa.Column(sa.Integer, nullable=False)
    mfg_locked = sa.Column(sa.Numeric, nullable=False, default=0)
    threat_locked = sa.Column(sa.Numeric, nullable=False, default=0)
    threat = sa.Column(sa.Integer)
    date_created = sa.Column(sa.String)


class ApcHashes(Base):
    __tablename__ = "apc_hashes"
    id = sa.Column(sa.BigInteger, primary_key=True)
    sha256 = sa.Column(sa.String, nullable=False, unique=True)
    created_ts = sa.Column(sa.DateTime, nullable=False, default=datetime.utcnow())
    last_updated_ts = sa.Column(sa.DateTime, nullable=False, default=datetime.utcnow())
    verdict = sa.Column(sa.Enum(ApcCategory))
    analysis_date = sa.Column(sa.DateTime)
